@@ -1021,6 +1021,8 @@ class Compound(Node, Registered):
                  get_type=None,
                  c_symbol_prefix=None,
                  disguised=False,
+                 opaque=False,
+                 pointer=False,
                  tag_name=None):
         Node.__init__(self, name)
         Registered.__init__(self, gtype_name, get_type)
@@ -1030,6 +1032,8 @@ class Compound(Node, Registered):
         self.fields = []
         self.constructors = []
         self.disguised = disguised
+        self.opaque = opaque
+        self.pointer = pointer
         self.gtype_name = gtype_name
         self.get_type = get_type
         self.c_symbol_prefix = c_symbol_prefix
@@ -1116,6 +1120,8 @@ class Record(Compound):
                  get_type=None,
                  c_symbol_prefix=None,
                  disguised=False,
+                 opaque=False,
+                 pointer=False,
                  tag_name=None):
         Compound.__init__(self, name,
                           ctype=ctype,
@@ -1123,10 +1129,18 @@ class Record(Compound):
                           get_type=get_type,
                           c_symbol_prefix=c_symbol_prefix,
                           disguised=disguised,
+                          opaque=opaque,
+                          pointer=pointer,
                           tag_name=tag_name)
         # If non-None, this record defines the FooClass C structure
         # for some Foo GObject (or similar for GInterface)
         self.is_gtype_struct_for = None
+        # If non-None, this record has a copy function for heap
+        # allocated instances
+        self.copy_func = None
+        # If non-None, this record has a free function for heap
+        # allocated instances
+        self.free_func = None
 
 
 class Union(Compound):
@@ -1137,6 +1151,8 @@ class Union(Compound):
                  get_type=None,
                  c_symbol_prefix=None,
                  disguised=False,
+                 opaque=False,
+                 pointer=False,
                  tag_name=None):
         Compound.__init__(self, name,
                           ctype=ctype,
@@ -1144,7 +1160,15 @@ class Union(Compound):
                           get_type=get_type,
                           c_symbol_prefix=c_symbol_prefix,
                           disguised=disguised,
+                          opaque=opaque,
+                          pointer=pointer,
                           tag_name=tag_name)
+        # If non-None, this union has a copy function for heap
+        # allocated instances
+        self.copy_func = None
+        # If non-None, this union has a free function for heap
+        # allocated instances
+        self.free_func = None
 
 
 class Boxed(Node, Registered):
@@ -1304,6 +1328,7 @@ class Property(Node):
             self.transfer = transfer
         self.setter = None
         self.getter = None
+        self.default_value = None
         self.parent = None  # A Class or Interface
 
 
