@@ -75,17 +75,17 @@ class GIRParser(object):
 
     def _find_first_child(self, node, name_or_names):
         if isinstance(name_or_names, str):
-            for child in node.getchildren():
+            for child in node:
                 if child.tag == name_or_names:
                     return child
         else:
-            for child in node.getchildren():
+            for child in node:
                 if child.tag in name_or_names:
                     return child
         return None
 
     def _find_children(self, node, name):
-        return [child for child in node.getchildren() if child.tag == name]
+        return [child for child in node if child.tag == name]
 
     def _get_current_file(self):
         if not self._filename_stack:
@@ -103,7 +103,7 @@ class GIRParser(object):
             raise SystemExit("%s: Incompatible version %s (supported: %s)" %
                              (self._get_current_file(), version, COMPATIBLE_GIR_VERSION))
 
-        for node in root.getchildren():
+        for node in root:
             if node.tag == _corens('include'):
                 self._parse_include(node)
             elif node.tag == _corens('package'):
@@ -138,6 +138,7 @@ class GIRParser(object):
             _corens('interface'): self._parse_object_interface,
             _corens('record'): self._parse_record,
             _corens('union'): self._parse_union,
+            _corens('docsection'): self._parse_doc_section,
             _glibns('boxed'): self._parse_boxed}
 
         if not self._types_only:
@@ -145,11 +146,16 @@ class GIRParser(object):
             parser_methods[_corens('function-macro')] = self._parse_function_macro
             parser_methods[_corens('function')] = self._parse_function
 
-        for node in ns.getchildren():
+        for node in ns:
             method = parser_methods.get(node.tag)
             if method is not None:
                 method(node)
 
+    def _parse_doc_section(self, node):
+        docsection = ast.DocSection(node.attrib["name"])
+        self._parse_generic_attribs(node, docsection)
+        self._namespace.append(docsection)
+
     def _parse_include(self, node):
         include = ast.Include(node.attrib['name'], node.attrib['version'])
         self._includes.add(include)
@@ -253,6 +259,9 @@ class GIRParser(object):
             is_abstract = node.attrib.get('abstract')
             is_abstract = is_abstract and is_abstract != '0'
             ctor_kwargs['is_abstract'] = is_abstract
+            is_final = node.attrib.get('final')
+            is_final = is_final and is_final != '0'
+            ctor_kwargs['is_final'] = is_final
         else:
             raise AssertionError(node)
 
@@ -370,9 +379,12 @@ class GIRParser(object):
                          detailed=node.attrib.get('detailed', '0') == '1',
                          action=node.attrib.get('action', '0') == '1',
                          no_hooks=node.attrib.get('no-hooks', '0') == '1')
+            func.emitter = node.attrib.get('emitter')
         else:
             assert False
 
+        func.set_property = node.attrib.get(_glibns('set-property'), None)
+        func.get_property = node.attrib.get(_glibns('get-property'), None)
         func.shadows = node.attrib.get('shadows', None)
         func.shadowed_by = node.attrib.get('shadowed-by', None)
         func.moved_to = node.attrib.get('moved-to', None)
@@ -413,7 +425,7 @@ class GIRParser(object):
     def _parse_fields(self, node, obj):
         res = []
         names = (_corens('field'), _corens('record'), _corens('union'), _corens('callback'))
-        for child in node.getchildren():
+        for child in node:
             if child.tag in names:
                 fieldobj = self._parse_field(child, obj)
                 res.append(fieldobj)
@@ -423,6 +435,8 @@ class GIRParser(object):
         compound = cls(node.attrib.get('name'),
                        ctype=node.attrib.get(_cns('type')),
                        disguised=node.attrib.get('disguised') == '1',
+                       opaque=node.attrib.get('opaque') == '1',
+                       pointer=node.attrib.get('pointer') == '1',
                        gtype_name=node.attrib.get(_glibns('type-name')),
                        get_type=node.attrib.get(_glibns('get-type')),
                        c_symbol_prefix=node.attrib.get(_cns('symbol-prefix')))
@@ -450,14 +464,20 @@ class GIRParser(object):
     def _parse_record(self, node, anonymous=False):
         struct = self._parse_compound(ast.Record, node)
         is_gtype_struct_for = node.attrib.get(_glibns('is-gtype-struct-for'))
+        copy_func = node.attrib.get('copy-function')
+        free_func = node.attrib.get('free-function')
         if is_gtype_struct_for is not None:
             struct.is_gtype_struct_for = self._namespace.type_from_name(is_gtype_struct_for)
+        struct.copy_func = copy_func
+        struct.free_func = free_func
         if not anonymous:
             self._namespace.append(struct)
         return struct
 
     def _parse_union(self, node, anonymous=False):
         union = self._parse_compound(ast.Union, node)
+        union.copy_func = node.attrib.get('copy-function')
+        union.free_func = node.attrib.get('free-function')
         if not anonymous:
             self._namespace.append(union)
         return union
@@ -595,15 +615,19 @@ class GIRParser(object):
                             node.attrib.get('construct') == '1',
                             node.attrib.get('construct-only') == '1',
                             node.attrib.get('transfer-ownership'))
-        self._parse_generic_attribs(node, prop)
+        prop.setter = node.attrib.get('setter')
+        prop.getter = node.attrib.get('getter')
+        prop.default_value = node.attrib.get('default-value')
         prop.parent = parent
+        self._parse_generic_attribs(node, prop)
         return prop
 
     def _parse_member(self, node):
         member = ast.Member(node.attrib['name'],
                             node.attrib['value'],
                             node.attrib.get(_cns('identifier')),
-                            node.attrib.get(_glibns('nick')))
+                            node.attrib.get(_glibns('nick')),
+                            node.attrib.get(_glibns('name')))
         self._parse_generic_attribs(node, member)
         return member
 
