@@ -119,6 +119,76 @@ invoke_error_quark (GModule *self, const char *symbol, GError **error)
   return sym ();
 }
 
+static char *
+value_transform_to_string (const GValue *value)
+{
+  GValue tmp = G_VALUE_INIT;
+  char *s = NULL;
+
+  g_value_init (&tmp, G_TYPE_STRING);
+
+  if (g_value_transform (value, &tmp))
+    {
+      const char *str = g_value_get_string (&tmp);
+
+      if (str != NULL)
+        s = g_strescape (str, NULL);
+    }
+
+  g_value_unset (&tmp);
+
+  return s;
+}
+
+/* A simpler version of g_strdup_value_contents(), but with stable
+ * output and less complex semantics
+ */
+static char *
+value_to_string (const GValue *value)
+{
+  if (value == NULL)
+    return NULL;
+
+  if (G_VALUE_HOLDS_STRING (value))
+    {
+      const char *s = g_value_get_string (value);
+
+      if (s == NULL)
+        return g_strdup ("NULL");
+
+      return g_strescape (s, NULL);
+    }
+  else
+    {
+      GType value_type = G_VALUE_TYPE (value);
+
+      switch (G_TYPE_FUNDAMENTAL (value_type))
+        {
+        case G_TYPE_BOXED:
+          if (g_value_get_boxed (value) == NULL)
+            return NULL;
+          else
+            return value_transform_to_string (value);
+          break;
+
+        case G_TYPE_OBJECT:
+          if (g_value_get_object (value) == NULL)
+            return NULL;
+          else
+            return value_transform_to_string (value);
+          break;
+
+        case G_TYPE_POINTER:
+          return NULL;
+
+        default:
+          return value_transform_to_string (value);
+        }
+    }
+
+  return NULL;
+}
+
 static void
 dump_properties (GType type, GOutputStream *out)
 {
@@ -147,9 +217,28 @@ dump_properties (GType type, GOutputStream *out)
       if (prop->owner_type != type)
 	continue;
 
-      escaped_printf (out, "    <property name=\"%s\" type=\"%s\" flags=\"%d\"/>\n",
-		      prop->name, g_type_name (prop->value_type), prop->flags);
+      const GValue *v = g_param_spec_get_default_value (prop);
+      char *default_value = value_to_string (v);
+
+      if (v != NULL && default_value != NULL)
+        {
+          escaped_printf (out, "    <property name=\"%s\" type=\"%s\" flags=\"%d\" default-value=\"%s\"/>\n",
+                          prop->name,
+                          g_type_name (prop->value_type),
+                          prop->flags,
+                          default_value);
+        }
+      else
+        {
+          escaped_printf (out, "    <property name=\"%s\" type=\"%s\" flags=\"%d\"/>\n",
+                          prop->name,
+                          g_type_name (prop->value_type),
+                          prop->flags);
+        }
+
+      g_free (default_value);
     }
+
   g_free (props);
 }
 
@@ -204,6 +293,7 @@ dump_signals (GType type, GOutputStream *out)
 	}
       goutput_write (out, "    </signal>\n");
     }
+  g_free (sig_ids);
 }
 
 static void
@@ -240,6 +330,12 @@ dump_object_type (GType type, const char *symbol, GOutputStream *out)
 
   if (G_TYPE_IS_ABSTRACT (type))
     escaped_printf (out, " abstract=\"1\"");
+
+#if GLIB_CHECK_VERSION (2, 70, 0)
+  if (G_TYPE_IS_FINAL (type))
+    escaped_printf (out, " final=\"1\"");
+#endif
+
   goutput_write (out, ">\n");
 
   interfaces = g_type_interfaces (type, &n_interfaces);
@@ -249,6 +345,8 @@ dump_object_type (GType type, const char *symbol, GOutputStream *out)
       escaped_printf (out, "    <implements name=\"%s\"/>\n",
 		      g_type_name (itype));
     }
+  g_free (interfaces);
+
   dump_properties (type, out);
   dump_signals (type, out);
   goutput_write (out, "  </class>\n");
@@ -280,6 +378,8 @@ dump_interface_type (GType type, const char *symbol, GOutputStream *out)
       escaped_printf (out, "    <prerequisite name=\"%s\"/>\n",
 		      g_type_name (itype));
     }
+  g_free (interfaces);
+
   dump_properties (type, out);
   dump_signals (type, out);
   goutput_write (out, "  </interface>\n");
@@ -306,7 +406,7 @@ dump_flags_type (GType type, const char *symbol, GOutputStream *out)
     {
       GFlagsValue *value = &(klass->values[i]);
 
-      escaped_printf (out, "    <member name=\"%s\" nick=\"%s\" value=\"%d\"/>\n",
+      escaped_printf (out, "    <member name=\"%s\" nick=\"%s\" value=\"%u\"/>\n",
 		      value->value_name, value->value_nick, value->value);
     }
   goutput_write (out, "  </flags>\n");
@@ -349,6 +449,11 @@ dump_fundamental_type (GType type, const char *symbol, GOutputStream *out)
   if (G_TYPE_IS_ABSTRACT (type))
     escaped_printf (out, " abstract=\"1\"");
 
+#if GLIB_CHECK_VERSION (2, 70, 0)
+  if (G_TYPE_IS_FINAL (type))
+    escaped_printf (out, " final=\"1\"");
+#endif
+
   if (G_TYPE_IS_INSTANTIATABLE (type))
     escaped_printf (out, " instantiatable=\"1\"");
 
@@ -379,6 +484,7 @@ dump_fundamental_type (GType type, const char *symbol, GOutputStream *out)
       escaped_printf (out, "    <implements name=\"%s\"/>\n",
 		      g_type_name (itype));
     }
+  g_free (interfaces);
   goutput_write (out, "  </fundamental>\n");
 }
 
@@ -471,11 +577,20 @@ g_irepository_dump (const char *arg, GError **error)
   input_file = g_file_new_for_path (args[0]);
   output_file = g_file_new_for_path (args[1]);
 
+  g_strfreev (args);
+
   input = g_file_read (input_file, NULL, error);
+  g_object_unref (input_file);
+
   if (input == NULL)
-    return FALSE;
+    {
+      g_object_unref (output_file);
+      return FALSE;
+    }
 
   output = g_file_replace (output_file, NULL, FALSE, 0, NULL, error);
+  g_object_unref (output_file);
+
   if (output == NULL)
     {
       g_input_stream_close (G_INPUT_STREAM (input), NULL, NULL);
